@@ -45,6 +45,11 @@
 //! }).for_each(|_| ());
 //!```
 //!
+//! ## Usage Caveat
+//! This crate is designed to clone all resources captured by the closure
+//! for each thread. To prevent unintended RAM usage, you may wrap
+//! large data structure using `Arc`.
+//!
 //! ## Sequential Consistency
 //! The output order is guaranteed to be the same as the upstream iterator,
 //! but the execution order is not sequential.
@@ -203,16 +208,16 @@
 //!
 mod iter_async;
 
+use crossbeam::channel::{bounded, unbounded, Receiver, Sender};
 pub use iter_async::*;
 use num_cpus;
 use std::iter::Enumerate;
 use std::sync::atomic::{AtomicBool, Ordering};
-use std::sync::mpsc::{channel, sync_channel, Receiver, Sender};
 use std::sync::{Arc, Mutex};
 use std::thread;
 use std::thread::JoinHandle;
 
-const MAX_SIZE_FOR_THREAD: usize = 10;
+const MAX_SIZE_FOR_THREAD: usize = 100;
 
 pub trait IntoParallelIteratorSync<R, T, TL, F>
 where
@@ -314,12 +319,12 @@ where
         let cpus = num_cpus::get();
         let iterator_stopper = Arc::new(AtomicBool::new(false));
         // worker master
-        let (task_register, task_order) = channel();
+        let (task_register, task_order) = unbounded();
         let tasks = Arc::new(Mutex::new(tasks.into_iter().enumerate()));
         let mut handles = Vec::with_capacity(cpus);
         let mut receivers = Vec::with_capacity(cpus);
         for thread_number in 0..cpus {
-            let (sender, receiver) = sync_channel(MAX_SIZE_FOR_THREAD);
+            let (sender, receiver) = bounded(MAX_SIZE_FOR_THREAD);
             let task = tasks.clone();
             let register = task_register.clone();
             let iterator_stopper = iterator_stopper.clone();
@@ -474,7 +479,11 @@ impl<R> Drop for ParIterSync<R> {
 
 #[cfg(test)]
 mod test_par_iter {
+    #[cfg(feature = "bench")]
+    extern crate test;
     use crate::IntoParallelIteratorSync;
+    #[cfg(feature = "bench")]
+    use test::Bencher;
 
     fn error_at_1000(test_vec: &Vec<i32>, a: i32) -> Result<i32, ()> {
         let n = test_vec.get(a as usize).unwrap().to_owned();
@@ -593,4 +602,14 @@ mod test_par_iter {
         }
         assert_eq!(count, 10)
     }
+
+    #[cfg(feature = "bench")]
+    #[bench]
+    fn bench_into_par_iter_sync(b: &mut Bencher) {
+        b.iter(|| {
+            (0..1_000_000)
+                .into_par_iter_sync(|a| Ok(a))
+                .for_each(|_| {})
+        });
+    }
 }
